
# q-expansion of newform 3240.2.a.r, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3240
weight = 2
poly_data = [3, -5, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-4, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0], [0, 0, 0], [1, 0, 0], [-1, -1, 1], [-1, 0, -1], [-1, 2, -1], [1, -2, -1], [-1, 0, 1], [-3, 3, 1], [-3, 2, 0], [-5, 0, 1], [1, 0, 1], [0, 0, 1], [1, -2, -1], [-6, 1, -4], [-4, 2, 0], [-2, -2, -2], [-2, -4, 1], [2, -5, 0], [-3, -2, 3], [4, 4, 0], [-4, 4, -2], [-7, -1, -1], [-11, 0, 0], [12, -2, 4], [-2, -2, 2], [-3, 6, -1], [-7, -3, 3], [10, -2, 5], [-6, -6, 0], [-4, 1, -6], [-8, 0, 2], [-8, 2, -2], [4, 0, 4], [-6, -2, -3], [-11, 0, 1], [10, -2, 6], [11, -2, -3], [-4, 7, 0], [6, -8, 6], [-3, 6, -1], [-1, -6, 0], [-5, -2, 3], [13, -6, 1], [-6, -2, 4], [-5, -8, 1], [-1, -4, -1], [-12, 1, 0], [-5, 8, -5], [-1, 6, 2], [-11, 8, -3], [-2, -2, -8], [-8, 6, -1], [-5, 6, -1], [10, 0, 4], [5, 2, -1], [6, 6, -3], [-11, 4, -1], [-1, 10, 1], [2, 6, 1], [-11, 5, -3], [-7, -2, -9], [-9, 7, -1], [7, -10, -1], [11, 4, 1], [1, -4, 3], [-12, -2, -4], [16, -10, -6], [-1, -10, 5], [1, -4, -6], [-8, 8, 4], [-1, 14, -5], [11, -2, 1], [16, -6, 8], [22, 0, 0], [15, -6, 5], [-10, -4, -1], [10, -4, -4], [10, -6, -2], [4, 8, -6], [-5, 4, 3], [4, 8, -2], [-6, -8, 4], [9, -6, -1], [-14, -6, 0], [2, 3, 2], [0, -18, 8], [19, -4, 3], [1, -10, -6], [-5, -4, -9], [1, -10, -1], [18, 8, 0], [17, 4, 5], [12, 6, 2], [-6, 14, 4], [14, -3, 2], [1, 12, -2], [-1, 2, -6], [12, 1, 6], [-7, 8, 0], [11, -1, 7], [22, 4, -4], [18, -1, 6], [-18, 0, -8], [-8, 16, -12], [4, 2, 0], [-17, -1, 7], [15, 0, -5], [30, -8, -4], [-28, 0, -6], [-18, 5, 0], [9, 2, -5], [14, 6, 0], [9, 4, -5], [-22, -6, 4], [12, 6, 3], [8, -5, -4], [-32, 7, 4], [-19, 0, 5], [-2, -6, 0], [-10, -4, -8], [1, 0, 11], [21, 4, 3], [19, -6, 1], [14, -8, 4], [16, 4, 11], [-3, 10, -4], [6, -6, 6], [-8, -15, 2], [6, 8, 4], [-16, 6, -8], [13, -7, -5], [7, 2, 1], [21, -10, 9], [-3, -14, 8], [-17, -14, 4], [-8, 4, -4], [43, -6, 1], [19, -12, -7], [10, -2, 10], [24, 4, -6], [-4, 0, 7], [-20, -5, 2], [-4, 5, 0], [-12, -4, 1], [15, 8, 5], [17, 0, -5], [19, -10, 15], [27, -10, 9], [-12, 1, 4], [-23, 6, 9], [-16, 12, 1], [-2, 3, -6], [-31, -10, -1], [22, 9, -6], [-20, -2, -4], [2, -12, 4], [-36, 12, 6], [-14, 10, 8], [7, -6, 8], [18, -9, 4], [9, -4, -9], [11, 17, 3], [1, 16, -1], [-9, 8, -11], [23, -9, 11], [-32, 6, -4], [-12, 6, -12], [-11, 6, -10], [-1, -4, 11], [12, 2, -2], [-41, 0, 2], [-15, 12, -7], [-12, -14, 2], [4, -20, 8], [-28, 6, -4], [-16, -2, 2], [-4, -14, 8], [-5, 11, -17], [6, -4, 7], [-5, 2, -15], [7, 2, -9], [-23, -6, -9], [17, 14, -7], [28, 15, -2], [-21, 12, 8], [4, -12, 8], [25, 11, -1], [6, -8, -7], [-12, -2, 0], [23, -8, -11], [10, 1, -8], [-5, -2, -9], [10, -6, -5], [-14, 1, 6], [-31, -14, 1], [-22, 6, 13], [-24, 10, 2], [-1, -12, -1], [39, 3, 7], [2, 4, 4], [21, -14, 7], [32, 4, -8], [-3, 18, -14], [14, -4, 22], [-11, 14, -11], [-21, 8, -5], [-39, 9, 1], [-5, 4, -14], [20, -2, 10], [22, 12, 8], [-34, 8, -8], [-28, 9, 4], [-11, 6, 15], [-1, -4, 7], [-4, 2, -17], [-9, 1, -1], [22, -4, -4], [-22, 5, -12], [18, -18, 12], [6, -8, -11], [-7, 16, -3], [-27, 8, 0], [9, -19, -5], [-4, 3, 6], [32, -10, 4], [-29, 4, -5], [13, -14, 7], [10, 3, -6], [-19, -4, -1], [-4, -6, -12], [47, 0, 3], [-7, -6, -9], [12, 2, -5], [16, 11, 4], [-9, 15, -3], [-4, 0, 7], [-27, 10, 9], [-15, 4, -5], [-41, 0, -13], [-7, 15, -1], [9, 2, 19], [10, 5, -2], [-27, 10, -8], [1, 14, -1], [15, -6, -9], [15, -7, -1], [11, -16, 5], [-11, 6, -23], [27, -23, 5], [-9, -12, -1], [9, 18, -12], [-39, 0, 1], [-18, 8, -11], [-10, 24, -6], [6, -12, -6], [28, 14, -12], [-12, -17, -10], [-25, 4, -1], [3, -22, 1], [4, -15, 12], [-56, 7, -6], [-14, 6, 18], [-36, 6, 4], [-3, 6, 5], [-2, 0, -12], [-34, 2, -2], [-21, 6, 10], [2, 27, -14], [-38, 10, 6], [-36, 6, -20], [-11, 27, 1], [-41, 6, -15], [-22, 22, 10], [-20, -12, 4], [26, -15, -10], [-6, -21, 0], [-26, 2, -2], [16, -24, 1], [21, 6, 15], [25, -14, 7], [17, -6, -5], [-27, -12, 1], [29, -12, 16], [-3, 5, -1], [-16, -8, 8], [-33, -12, 9], [26, 8, 10], [12, -12, -16], [42, -2, 15], [0, -14, -8], [3, 26, -15], [-11, -8, 1], [-21, -6, 7], [7, -4, 15], [-2, -14, -3], [-13, 10, -23], [-16, -10, -10], [-38, 16, -8], [39, 2, 13], [9, -28, 5], [-4, 2, -14], [-15, 30, -7], [-51, 12, -15], [2, 10, -10], [-15, 2, -11], [31, -1, -9], [7, -10, -2], [-25, -4, 9], [20, 0, 12], [63, -6, 5], [-29, 12, 4], [-22, 10, 10], [33, -13, 11], [-5, 2, 9], [-25, -10, 4], [-26, 28, 8], [15, -20, 15], [-38, 18, 10], [-16, -2, -16], [8, 2, -6], [11, -16, 5], [-20, 0, 10], [-5, 8, 7], [-12, -18, -8], [30, 20, 3], [-5, 14, 3], [-57, -1, 1], [-8, 11, -8], [-14, 22, -6], [34, 2, 3], [3, -18, 5], [24, -2, 14], [11, -10, 13], [-6, -22, 2], [-12, -9, -22], [4, -10, -5], [-22, 14, -16], [18, 6, 17], [-19, 2, -1], [16, -16, 0], [-32, 12, -8], [-22, -10, -15], [70, -2, 2], [15, 30, -11], [18, -2, -20], [10, -16, -16], [-41, 7, -9], [-6, -12, 4], [16, -24, -20], [42, 6, -2], [-37, 24, -23], [-24, 26, -3], [1, 5, -13], [5, -32, 6], [-3, 10, 19], [-30, 18, -24], [50, -16, -14], [-14, 20, 2], [38, -17, 24], [4, 6, 2], [55, 4, 4], [-7, -16, 17], [65, -8, 13], [-11, -10, -21], [41, -10, -9], [-9, 2, 3], [-58, 1, -6], [20, -4, -6], [-25, -38, 11], [-28, -30, 6], [7, -1, 11], [-20, -34, 11], [18, 8, 22], [-38, 26, -12], [25, -32, 13], [-46, -10, 8], [64, -10, -2], [16, 4, 7], [-21, 4, 19], [-11, 0, -8], [19, -6, 23], [-44, -1, -8], [1, -12, -5], [-5, 14, -25], [49, 18, 3], [-49, 18, 3], [-30, 16, 18], [39, -23, -1], [27, -13, 5], [40, -16, 19], [39, -10, -17], [56, -6, 24], [-19, 10, 19], [-32, 8, 14], [-42, 14, -6], [12, -22, -4], [-2, 10, 6], [20, -4, 30], [36, 6, -8], [12, 14, -1], [-8, -16, -2], [-19, -14, 11], [57, 6, -1], [-60, -12, -3], [-7, 36, 1], [-20, 12, -12], [33, -18, -12], [-57, 13, -1], [-14, 38, -14], [-6, -22, -12], [-20, 6, -14], [-50, 17, -4], [-30, 26, -4], [-27, -18, -11], [1, 14, 2], [13, -2, -5], [1, 30, -1], [-16, 4, 8], [24, 29, -12], [36, -16, 29], [22, -2, 0], [-33, 14, -15], [2, 2, -2], [-17, -6, -3], [36, -12, 36], [-66, -1, 10], [48, -4, -6], [1, -4, 21], [36, 4, 6]]
